mod command;
pub mod errors;

use command::*;
use errors::*;

use clap::Parser;
use std::fmt::Display;
use std::io;
use std::path::PathBuf;

#[derive(Parser)]
pub struct Opt {
    #[arg(hide = true)] // TODO: hidden option for debug? or usefull for everyone?
    pub config_file: Option<PathBuf>,

    #[arg(short, long)]
    pub quiet: bool, // TODO: use clap_verbosity_flag instead

    #[arg(short, long)]
    pub steps: Vec<String>,
}

enum Status {
    /// All steps asked were successful
    Ok,
    Err(MyError),
}

impl Display for Status {
    fn fmt(&self, f: &mut std::fmt::Formatter<'_>) -> std::fmt::Result {
        match self {
            Status::Ok => write!(f, "Ok"),
            Status::Err(e) => write!(f, "Error: {}", e),
            // State::Fetch(e) => write!(f, "Fetch error: {}", e),
            // State::Compile(e) => write!(f, "Compile error: {}", e),
            // State::Install(e) => write!(f, "Install error: {}", e),
        }
    }
}

impl<T> From<Result<T>> for Status {
    fn from(value: Result<T>) -> Self {
        match value {
            Ok(_) => Status::Ok,
            Err(e) => Status::Err(e),
        }
    }
}

pub struct Summary {
    // TODO: Go back to vectors to keep order?
    status: Vec<(String, Status)>,
}

impl Display for Summary {
    fn fmt(&self, f: &mut std::fmt::Formatter<'_>) -> std::fmt::Result {
        writeln!(f, "Summary:")?;
        for (name, status) in &self.status {
            // TODO: also print version before/after, update time

            writeln!(f, "\t{}\t{}", name, status)?;
        }
        Ok(())
    }
}

pub fn run(opt: &Opt) -> io::Result<Summary> {
    let updater = Updater::from_config(opt)?;

    if updater.systems.is_empty() {
        let package_folder = get_packages_folder(&opt)?;
        return Err(io::Error::new(
            io::ErrorKind::Other,
            format!(
                "No package found in configuration folder. Add them in: {}",
                package_folder.display()
            ),
        ));
    }

    Ok(updater.update_all(opt))
}
